import greenfoot.*;  // （World、Actor、GreenfootImage、GreenfootSound、Greenfoot及MouseInfo）

/**
 * 有地雷的田野。
 * 
 * @author 依瑪貓
 * @version 2013-04-11
 */
public class MineField extends World {
    
    /**
     * 建立新的地雷田野。
     * 
     */
    public MineField() {    
        // 建立9x9方格的新場景，方格大小為16x16像素。
        super(9, 9, 16);
        addBombs(10);
        addCounts();
        addCovers();
    }
    
    /**
     * 每個回合執行的動作。
     * 
     */
    public void act() {
        checkGameOver();
    }
    
    /**
     * 檢查遊戲結束。
     * 
     */
    public void checkGameOver() {
        int numBombs = getObjects(Bomb.class).size();
        int numCovers = getObjects(Cover.class).size();
        int numFlags = getObjects(Flag.class).size();
        if (numBombs == numCovers && numCovers == numFlags) {
            Greenfoot.stop();
        }
    }
    
    /**
     * 加上地雷。
     * 
     * @param count 地雷的數目
     */
    private void addBombs(int count) {
        for (int i = 0; i < count; i++) {
            int x = -1, y = -1;
            do {
                x = Greenfoot.getRandomNumber(getWidth());
                y = Greenfoot.getRandomNumber(getHeight());
            } while (getObjectsAt(x, y, Bomb.class).size() > 0);
            addObject(new Bomb(), x, y);
        }
    }
    
    /**
     * 加上周圍地雷的提示數字。
     * 
     */
    private void addCounts() {
        for (int x = 0; x < getWidth(); x++) {
            for (int y = 0; y < getHeight(); y++) {
                if (getObjectsAt(x, y, Bomb.class).size() == 0) {
                    addObject(new Count(), x, y);
                }
            }
        }
    }
    
    /**
     * 加上地雷的蓋板。
     * 
     */
    private void addCovers() {
        for (int x = 0; x < getWidth(); x++) {
            for (int y = 0; y < getHeight(); y++) {
                addObject(new Cover(), x, y);
            }
        }
    }
}